@@ -606,8 +606,10 @@ var specialsKeys = map[specialKey]rune{
 	{5, '~'}:     KeyPgUp,
 	{6, '~'}:     KeyPgDown,
 	{1, 'F'}:     KeyEnd,
+	{4, '~'}:     KeyEnd,
 	{8, '~'}:     KeyEnd,
 	{1, 'H'}:     KeyHome,
+	{1, '~'}:     KeyHome,
 	{7, '~'}:     KeyHome,
 	{57358, 'u'}: KeyCapsLock,
 	{57359, 'u'}: KeyScrollLock,
