@@ -32,8 +32,13 @@ type capabilities struct {
 	colorThemeUpdates  bool
 	reportSizeChars    bool
 	reportSizePixels   bool
+	osc4               bool
+	osc10              bool
 	osc11              bool
 	osc176             bool
+	inBandResize       bool
+	explicitWidth      bool
+	sgrPixels          bool
 }
 
 type cursorState struct {
@@ -69,6 +74,13 @@ type Options struct {
 	// CSIuBitMask is the bit mask to use for CSIu key encoding, when
 	// available. This has no effect if DisableKittyKeyboard is true
 	CSIuBitMask CSIuBitMask
+
+	// WithConsole provides the ability to use a custom console.
+	WithConsole console.Console
+
+	// EnableSGRPixels provides pixel level precision of mouse movement. This has
+	// no effect if DisableMouse is true
+	EnableSGRPixels bool
 }
 
 type CSIuBitMask int
@@ -113,11 +125,15 @@ type Vaxis struct {
 	refresh          bool
 	kittyFlags       int
 	disableMouse     bool
+	chFg             chan string
 	chBg             chan string
+	chColor          chan string
+	userCursorStyle  CursorStyle
 
 	xtwinops bool
 
-	withTty string
+	withTty     string
+	withConsole console.Console
 
 	termID terminalID
 
@@ -126,6 +142,8 @@ type Vaxis struct {
 
 	mu     sync.Mutex
 	resize int32
+
+	noSignals bool
 }
 
 // New creates a new [Vaxis] instance. Calling New will query the underlying
@@ -176,22 +194,27 @@ func New(opts Options) (*Vaxis, error) {
 		vx.disableMouse = true
 	}
 
+	vx.noSignals = opts.NoSignals
+
 	var tgts []*os.File
-	switch opts.WithTTY {
-	case "":
-		f, err := os.OpenFile("/dev/tty", os.O_RDWR, 0)
-		if err != nil {
-			tgts = []*os.File{os.Stderr, os.Stdout, os.Stdin}
-			break
-		}
-		tgts = []*os.File{f, os.Stderr, os.Stdout, os.Stdin}
-	default:
+
+	switch {
+	case opts.WithConsole != nil:
+		vx.withConsole = opts.WithConsole
+	case opts.WithTTY != "":
 		vx.withTty = opts.WithTTY
 		f, err := os.OpenFile(opts.WithTTY, os.O_RDWR, 0)
 		if err != nil {
 			return nil, err
 		}
 		tgts = []*os.File{f}
+	default:
+		f, err := os.OpenFile("/dev/tty", os.O_RDWR, 0)
+		if err != nil {
+			tgts = []*os.File{os.Stderr, os.Stdout, os.Stdin}
+			break
+		}
+		tgts = []*os.File{f, os.Stderr, os.Stdout, os.Stdin}
 	}
 
 	vx.queue = make(chan Event, opts.EventQueueSize)
@@ -204,7 +227,10 @@ func New(opts Options) (*Vaxis, error) {
 	vx.chQuit = make(chan bool)
 	vx.chSizeDone = make(chan bool, 1)
 	vx.charCache = make(map[string]int, 256)
+	vx.chFg = make(chan string, 1)
 	vx.chBg = make(chan string, 1)
+	vx.chColor = make(chan string, 1)
+
 	err = vx.openTty(tgts)
 	if err != nil {
 		return nil, err
@@ -223,59 +249,107 @@ outer:
 				break outer
 			case capabilitySixel:
 				log.Info("[capability] Sixel graphics")
+				vx.mu.Lock()
 				vx.caps.sixels = true
 				if vx.graphicsProtocol < sixelGraphics {
 					vx.graphicsProtocol = sixelGraphics
 				}
+				vx.mu.Unlock()
+			case capabilityOsc4:
+				log.Info("[capability] OSC 4 supported")
+				vx.mu.Lock()
+				vx.caps.osc4 = true
+				vx.mu.Unlock()
+			case capabilityOsc10:
+				log.Info("[capability] OSC 10 supported")
+				vx.mu.Lock()
+				vx.caps.osc10 = true
+				vx.mu.Unlock()
 			case capabilityOsc11:
-				vx.caps.osc11 = true
 				log.Info("[capability] OSC 11 supported")
+				vx.mu.Lock()
+				vx.caps.osc11 = true
+				vx.mu.Unlock()
 			case synchronizedUpdates:
 				log.Info("[capability] Synchronized updates")
+				vx.mu.Lock()
 				vx.caps.synchronizedUpdate = true
+				vx.mu.Unlock()
 			case unicodeCoreCap:
 				log.Info("[capability] Unicode core")
+				vx.mu.Lock()
 				vx.caps.unicodeCore = true
+				vx.mu.Unlock()
 			case notifyColorChange:
 				log.Info("[capability] Color theme notifications")
+				vx.mu.Lock()
 				vx.caps.colorThemeUpdates = true
+				vx.mu.Unlock()
 			case kittyKeyboard:
 				log.Info("[capability] Kitty keyboard")
 				if opts.DisableKittyKeyboard {
 					continue
 				}
+				vx.mu.Lock()
 				vx.caps.kittyKeyboard = true
+				vx.mu.Unlock()
 			case styledUnderlines:
-				vx.caps.styledUnderlines = true
 				log.Info("[capability] Styled underlines")
+				vx.mu.Lock()
+				vx.caps.styledUnderlines = true
+				vx.mu.Unlock()
 			case truecolor:
-				vx.caps.rgb = true
 				log.Info("[capability] RGB")
+				vx.mu.Lock()
+				vx.caps.rgb = true
+				vx.mu.Unlock()
 			case kittyGraphics:
 				log.Info("[capability] Kitty graphics supported")
+				vx.mu.Lock()
 				vx.caps.kittyGraphics = true
 				if vx.graphicsProtocol < kitty {
 					vx.graphicsProtocol = kitty
 				}
+				vx.mu.Unlock()
 			case textAreaPix:
-				vx.caps.reportSizePixels = true
 				log.Info("[capability] Report screen size: pixels")
+				vx.mu.Lock()
+				vx.caps.reportSizePixels = true
+				vx.mu.Unlock()
 			case textAreaChar:
-				vx.caps.reportSizeChars = true
 				log.Info("[capability] Report screen size: characters")
+				vx.mu.Lock()
+				vx.caps.reportSizeChars = true
+				vx.mu.Unlock()
 			case appID:
+				log.Info("[capability] OSC 176 supported")
+				vx.mu.Lock()
 				vx.caps.osc176 = true
 				vx.appIDLast = ev
-				log.Info("[capability] OSC 176 supported")
+				vx.mu.Unlock()
 			case terminalID:
+				vx.mu.Lock()
 				vx.termID = ev
+				vx.mu.Unlock()
+			case inBandResizeEvents:
+				vx.mu.Lock()
+				vx.caps.inBandResize = true
+				vx.mu.Unlock()
+			case capabilitySgrPixels:
+				log.Info("[capability] SGR Pixels supported")
+				if !opts.EnableSGRPixels {
+					continue
+				}
+				vx.mu.Lock()
+				vx.caps.sgrPixels = true
+				vx.mu.Unlock()
 			}
 		}
 	}
 
 	vx.enterAltScreen()
 	vx.enableModes()
-	if !opts.NoSignals {
+	if !vx.noSignals {
 		vx.setupSignals()
 	}
 	vx.applyQuirks()
@@ -310,6 +384,8 @@ outer:
 	vx.screenNext.resize(ws.Cols, ws.Rows)
 	vx.screenLast.resize(ws.Cols, ws.Rows)
 	vx.winSize = ws
+	// Set the next style to be a CursorBlock by default.
+	vx.cursorNext.style = CursorBlock
 	vx.PostEvent(vx.winSize)
 	return vx, nil
 }
@@ -325,6 +401,13 @@ func (vx *Vaxis) PostEvent(ev Event) {
 	}
 }
 
+// PostEventBlocking inserts an event into the [Vaxis] event loop. The call will
+// block if the queue is full. This method should only be used from a different
+// goroutine than the main thread.
+func (vx *Vaxis) PostEventBlocking(ev Event) {
+	vx.queue <- ev
+}
+
 // SyncFunc queues a function to be called from the main thread. vaxis will call
 // the function when the event is received in the main thread either through
 // PollEvent or Events. A Redraw event will be sent to the host application
@@ -664,9 +747,11 @@ outerNew:
 				next.Width = vx.characterWidth(next.Grapheme)
 			}
 
-			switch next.Width {
-			case 0:
+			switch {
+			case next.Width == 0:
 				_, _ = vx.tw.WriteString(" ")
+			case next.Width > 1 && vx.caps.explicitWidth:
+				_, _ = fmt.Fprintf(vx.tw, explicitWidth, next.Width, next.Grapheme)
 			default:
 				_, _ = vx.tw.WriteString(next.Grapheme)
 			}
@@ -697,25 +782,25 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 		if vx.pastePending {
 			key.EventType = EventPaste
 		}
-		vx.PostEvent(key)
+		vx.PostEventBlocking(key)
 	case ansi.C0:
 		key := decodeKey(seq)
 		if vx.pastePending {
 			key.EventType = EventPaste
 		}
-		vx.PostEvent(key)
+		vx.PostEventBlocking(key)
 	case ansi.ESC:
 		key := decodeKey(seq)
 		if vx.pastePending {
 			key.EventType = EventPaste
 		}
-		vx.PostEvent(key)
+		vx.PostEventBlocking(key)
 	case ansi.SS3:
 		key := decodeKey(seq)
 		if vx.pastePending {
 			key.EventType = EventPaste
 		}
-		vx.PostEvent(key)
+		vx.PostEventBlocking(key)
 	case ansi.CSI:
 		switch seq.Final {
 		case 'c':
@@ -723,17 +808,17 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				for _, ps := range seq.Parameters {
 					switch ps[0] {
 					case 4:
-						vx.PostEvent(capabilitySixel{})
+						vx.PostEventBlocking(capabilitySixel{})
 					}
 				}
-				vx.PostEvent(primaryDeviceAttribute{})
+				vx.PostEventBlocking(primaryDeviceAttribute{})
 				return
 			}
 		case 'I':
-			vx.PostEvent(FocusIn{})
+			vx.PostEventBlocking(FocusIn{})
 			return
 		case 'O':
-			vx.PostEvent(FocusOut{})
+			vx.PostEventBlocking(FocusOut{})
 			return
 		case 'R':
 			// KeyF1 or DSRCPR
@@ -762,7 +847,7 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				switch seq.Parameters[0][0] {
 				case 2:
 					if seq.Parameters[1][0] == 0 {
-						vx.PostEvent(capabilitySixel{})
+						vx.PostEventBlocking(capabilitySixel{})
 					}
 				}
 				return
@@ -775,7 +860,7 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				switch seq.Parameters[0][0] {
 				case colorThemeResp: // 997
 					m := ColorThemeMode(seq.Parameters[1][0])
-					vx.PostEvent(ColorThemeUpdate{
+					vx.PostEventBlocking(ColorThemeUpdate{
 						Mode: m,
 					})
 				}
@@ -788,6 +873,15 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				return
 			}
 			switch seq.Parameters[0][0] {
+			case 1016:
+				if len(seq.Parameters) < 2 {
+					log.Error("not enough DECRPM params")
+					return
+				}
+				switch seq.Parameters[1][0] {
+				case 1, 2:
+					vx.PostEventBlocking(capabilitySgrPixels{})
+				}
 			case 2026:
 				if len(seq.Parameters) < 2 {
 					log.Error("not enough DECRPM params")
@@ -795,7 +889,7 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				}
 				switch seq.Parameters[1][0] {
 				case 1, 2:
-					vx.PostEvent(synchronizedUpdates{})
+					vx.PostEventBlocking(synchronizedUpdates{})
 				}
 			case 2027:
 				if len(seq.Parameters) < 2 {
@@ -804,7 +898,7 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				}
 				switch seq.Parameters[1][0] {
 				case 1, 2:
-					vx.PostEvent(unicodeCoreCap{})
+					vx.PostEventBlocking(unicodeCoreCap{})
 				}
 			case 2031:
 				if len(seq.Parameters) < 2 {
@@ -813,13 +907,13 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				}
 				switch seq.Parameters[1][0] {
 				case 1, 2:
-					vx.PostEvent(notifyColorChange{})
+					vx.PostEventBlocking(notifyColorChange{})
 				}
 			}
 			return
 		case 'u':
 			if len(seq.Intermediate) == 1 && seq.Intermediate[0] == '?' {
-				vx.PostEvent(kittyKeyboard{})
+				vx.PostEventBlocking(kittyKeyboard{})
 				return
 			}
 		case '~':
@@ -831,22 +925,22 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				switch seq.Parameters[0][0] {
 				case 200:
 					vx.pastePending = true
-					vx.PostEvent(PasteStartEvent{})
+					vx.PostEventBlocking(PasteStartEvent{})
 					return
 				case 201:
 					vx.pastePending = false
-					vx.PostEvent(PasteEndEvent{})
+					vx.PostEventBlocking(PasteEndEvent{})
 					return
 				}
 			}
 		case 'M', 'm':
-			mouse, ok := parseMouseEvent(seq)
+			mouse, ok := parseMouseEvent(seq, vx.winSize, vx.caps.sgrPixels)
 			if ok {
-				vx.PostEvent(mouse)
+				vx.PostEventBlocking(mouse)
 			}
 			return
 		case 't':
-			if len(seq.Parameters) != 3 {
+			if len(seq.Parameters) < 3 {
 				log.Error("[CSI] unknown sequence: %s", seq)
 				return
 			}
@@ -856,26 +950,49 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 			w := seq.Parameters[2][0]
 			switch typ {
 			case 4:
+				vx.mu.Lock()
 				vx.nextSize.XPixel = w
 				vx.nextSize.YPixel = h
-				if !vx.caps.reportSizePixels {
+				report := vx.caps.reportSizePixels
+				vx.mu.Unlock()
+				if !report {
 					// Gate on this so we only report this
 					// once at startup
-					vx.PostEvent(textAreaPix{})
+					vx.PostEventBlocking(textAreaPix{})
 					return
 				}
 			case 8:
+				vx.mu.Lock()
 				vx.nextSize.Cols = w
 				vx.nextSize.Rows = h
-				if !vx.caps.reportSizeChars {
+				report := vx.caps.reportSizeChars
+				vx.mu.Unlock()
+				if !report {
 					// Gate on this so we only report this
 					// once at startup. This also means we
 					// can set the size directly and won't
 					// have race conditions
-					vx.PostEvent(textAreaChar{})
+					vx.PostEventBlocking(textAreaChar{})
 					return
 				}
 				vx.chSizeDone <- true
+			case 48:
+				// CSI <type> ; <height> ; <width> ; <height_pix> ; <width_pix> t
+				switch len(seq.Parameters) {
+				case 5:
+					atomicStore(&vx.resize, true)
+					vx.mu.Lock()
+					vx.nextSize.Cols = w
+					vx.nextSize.Rows = h
+					vx.nextSize.YPixel = seq.Parameters[3][0]
+					vx.nextSize.XPixel = seq.Parameters[4][0]
+					resize := vx.caps.inBandResize
+					vx.mu.Unlock()
+					if !resize {
+						vx.PostEventBlocking(inBandResizeEvents{})
+					}
+					vx.Resize()
+				}
 			}
 			return
 		}
@@ -884,7 +1001,7 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 		if vx.pastePending {
 			key.EventType = EventPaste
 		}
-		vx.PostEvent(key)
+		vx.PostEventBlocking(key)
 	case ansi.DCS:
 		switch seq.Final {
 		case 'r':
@@ -906,10 +1023,29 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				}
 				switch vals[0] {
 				case hexEncode("Smulx"):
-					vx.PostEvent(styledUnderlines{})
+					vx.PostEventBlocking(styledUnderlines{})
 				case hexEncode("RGB"):
-					vx.PostEvent(truecolor{})
+					vx.PostEventBlocking(truecolor{})
 				}
+			case '$':
+				// DECRQSS response (DECRPSS)
+
+				// DECSCUSR (user cursor style)
+				if strings.HasSuffix(string(seq.Data), " q") {
+					// Convert the rune into a digit
+					cursorStyle := seq.Data[0]
+					// Valid cursor styles are 0-6
+					if cursorStyle < '0' || cursorStyle > '6' {
+						log.Warn("invalid DECSCUSR: %d", cursorStyle)
+						return
+					}
+					log.Debug("User cursor style discovered: %v",
+						CursorStyle(cursorStyle-0x30))
+					vx.mu.Lock()
+					vx.userCursorStyle = CursorStyle(cursorStyle - 0x30)
+					vx.mu.Unlock()
+				}
+
 			}
 		case '|':
 			if len(seq.Intermediate) < 1 {
@@ -920,10 +1056,10 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 				if string(seq.Data) == hexEncode("~VTE") {
 					// VTE supports styled underlines but
 					// doesn't respond to XTGETTCAP
-					vx.PostEvent(styledUnderlines{})
+					vx.PostEventBlocking(styledUnderlines{})
 				}
 			case '>':
-				vx.PostEvent(terminalID(seq.Data))
+				vx.PostEventBlocking(terminalID(seq.Data))
 			}
 		}
 	case ansi.APC:
@@ -931,19 +1067,33 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 			return
 		}
 		if strings.HasPrefix(seq.Data, "G") {
-			vx.PostEvent(kittyGraphics{})
+			vx.PostEventBlocking(kittyGraphics{})
 		}
 	case ansi.OSC:
-		if strings.HasPrefix(string(seq.Payload), "11") {
+		if strings.HasPrefix(string(seq.Payload), "4") {
 			// If we are here and don't know that the host terminal
 			// supports the sequence, it means we are handling the response
 			// to the initial query and thus nobody is expecting its actual
 			// content. In this case, we don't want to fill the channel buffer
 			// as no one will clear it.
+			if vx.CanReportColor() {
+				vx.chColor <- string(seq.Payload)
+			}
+			vx.PostEventBlocking(capabilityOsc4{})
+		}
+		if strings.HasPrefix(string(seq.Payload), "10") {
+			// Similar to OSC 4
+			if vx.CanReportForegroundColor() {
+				vx.chFg <- string(seq.Payload)
+			}
+			vx.PostEventBlocking(capabilityOsc10{})
+		}
+		if strings.HasPrefix(string(seq.Payload), "11") {
+			// Similar to OSC 4
 			if vx.CanReportBackgroundColor() {
 				vx.chBg <- string(seq.Payload)
 			}
-			vx.PostEvent(capabilityOsc11{})
+			vx.PostEventBlocking(capabilityOsc11{})
 		}
 		if strings.HasPrefix(string(seq.Payload), "52") {
 			vals := strings.Split(string(seq.Payload), ";")
@@ -974,6 +1124,58 @@ func (vx *Vaxis) handleSequence(seq ansi.Sequence) {
 	}
 }
 
+// QueryColor queries the host terminal for an indexed color and returns
+// it as an instance of an RGB vaxis.Color. If the host terminal doesn't
+// support this, Color(0) is returned instead. Make sure not to run this
+// in the same goroutine as Vaxis runs in or deadlock will occur.
+func (vx *Vaxis) QueryColor(c Color) Color {
+	if !vx.CanReportColor() {
+		return Color(0)
+	}
+	p := c.Params()
+	if len(p) == 3 {
+		// If an RGB color was passed, return it as is.
+		return c
+	}
+	if len(p) != 1 {
+		return Color(0)
+	}
+	vx.tw.WriteStringLocked(tparm(osc4, p[0]))
+	resp := <-vx.chColor
+	var r, g, b int
+	prefix := fmt.Sprintf("4;%v;", p[0])
+	_, err := fmt.Sscanf(resp, prefix+"rgb:%x/%x/%x", &r, &g, &b)
+	if err != nil {
+		log.Error("QueryColor: failed to parse the OSC 4 response: %s", err)
+		return Color(0)
+	}
+	// The returned value can in principle be 16 bits per channel, however
+	// we are not aware of any terminal that would do this, foot for
+	// instance just repeats the same 8 bits twice. Hence we only take the
+	// lower 8 bits.
+	return RGBColor(uint8(r), uint8(g), uint8(b))
+}
+
+// QueryForeground queries the host terminal for foreground color and returns
+// it as an instance of vaxis.Color. If the host terminal doesn't support this,
+// Color(0) is returned instead. Make sure not to run this in the same
+// goroutine as Vaxis runs in or deadlock will occur.
+func (vx *Vaxis) QueryForeground() Color {
+	if !vx.CanReportForegroundColor() {
+		return Color(0)
+	}
+	vx.tw.WriteStringLocked(osc10)
+	resp := <-vx.chFg
+	var r, g, b int
+	_, err := fmt.Sscanf(resp, "10;rgb:%x/%x/%x", &r, &g, &b)
+	if err != nil {
+		log.Error("QueryForeground: failed to parse the OSC 10 response: %s", err)
+		return Color(0)
+	}
+	// Similar to QueryColor above.
+	return RGBColor(uint8(r), uint8(g), uint8(b))
+}
+
 // QueryBackground queries the host terminal for background color and returns
 // it as an instance of vaxis.Color. If the host terminal doesn't support this,
 // Color(0) is returned instead. Make sure not to run this in the same
@@ -982,7 +1184,7 @@ func (vx *Vaxis) QueryBackground() Color {
 	if !vx.CanReportBackgroundColor() {
 		return Color(0)
 	}
-	vx.tw.WriteString(osc11)
+	vx.tw.WriteStringLocked(osc11)
 	resp := <-vx.chBg
 	var r, g, b int
 	_, err := fmt.Sscanf(resp, "11;rgb:%x/%x/%x", &r, &g, &b)
@@ -990,10 +1192,7 @@ func (vx *Vaxis) QueryBackground() Color {
 		log.Error("QueryBackground: failed to parse the OSC 11 response: %s", err)
 		return Color(0)
 	}
-	// The returned value can in principle be 16 bits per channel, however
-	// we are not aware of any terminal that would do this, foot for
-	// instance just repeats the same 8 bits twice. Hence we only take the
-	// lower 8 bits.
+	// Similar to QueryColor above.
 	return RGBColor(uint8(r), uint8(g), uint8(b))
 }
 
@@ -1008,20 +1207,38 @@ func (vx *Vaxis) sendQueries() {
 		vx.PostEvent(truecolor{})
 	}
 
+	_, _ = vx.tw.WriteString(userCursorStyle)
 	_, _ = vx.tw.WriteString(decrqm(synchronizedUpdate))
 	_, _ = vx.tw.WriteString(decrqm(unicodeCore))
 	_, _ = vx.tw.WriteString(decrqm(colorThemeUpdates))
+	_, _ = vx.tw.WriteString(decrqm(mouseSGRPixels))
+	// We blindly enable in band resize. We get a response immediately if it
+	// is supported
+	_, _ = vx.tw.WriteString(decset(inBandResize))
 	_, _ = vx.tw.WriteString(xtversion)
 	_, _ = vx.tw.WriteString(kittyKBQuery)
 	_, _ = vx.tw.WriteString(kittyGquery)
 	_, _ = vx.tw.WriteString(xtsmSixelGeom)
-	// Can the terminal report it's own size?
+	// Can the terminal report its own size?
 	_, _ = vx.tw.WriteString(textAreaSize)
 
+	// Explicit width query
+	_, _ = vx.tw.WriteString("\x1b[H")
+	_, _ = fmt.Fprintf(vx.tw, explicitWidth, 1, " ")
+	_, col := vx.CursorPosition()
+	if col == 1 {
+		log.Debug("[capability] explicit width supported")
+		vx.mu.Lock()
+		vx.caps.explicitWidth = true
+		vx.mu.Unlock()
+	}
+
 	// Query some terminfo capabilities
 	// Just another way to see if we have RGB support
 	_, _ = vx.tw.WriteString(xtgettcap("RGB"))
-	// Does the terminal respond to OSC 11 queries?
+	// Does the terminal respond to OSC 4/10/11 queries?
+	_, _ = vx.tw.WriteString(tparm(osc4, 1))
+	_, _ = vx.tw.WriteString(osc10)
 	_, _ = vx.tw.WriteString(osc11)
 	// Back up the current app ID
 	_, _ = vx.tw.WriteString(getAppID)
@@ -1049,8 +1266,9 @@ func (vx *Vaxis) enableModes() {
 	if vx.caps.sixels {
 		_, _ = vx.tw.WriteString(decset(sixelScrolling))
 	}
-	// Mode 2027, unicode segmentation (for correct emoji/wc widths)
-	if vx.caps.unicodeCore {
+	// Mode 2027, unicode segmentation (for correct emoji/wc widths). We
+	// only enable if we don't also have explicitWidth
+	if vx.caps.unicodeCore && !vx.caps.explicitWidth {
 		_, _ = vx.tw.WriteString(decset(unicodeCore))
 	}
 
@@ -1060,7 +1278,11 @@ func (vx *Vaxis) enableModes() {
 		// Let's query the current mode also
 		_, _ = vx.tw.WriteString(tparm(dsr, colorThemeReq))
 	}
+	if vx.caps.inBandResize {
+		_, _ = vx.tw.WriteString(decset(inBandResize))
+	}
 
+	_, _ = vx.tw.WriteString(decset(mouseFocusEvents)) // window focus events
 	// TODO: query for bracketed paste support?
 	_, _ = vx.tw.WriteString(decset(bracketedPaste)) // bracketed paste
 	_, _ = vx.tw.WriteString(decset(cursorKeys))     // application cursor keys
@@ -1073,8 +1295,10 @@ func (vx *Vaxis) enableModes() {
 	if !vx.disableMouse {
 		_, _ = vx.tw.WriteString(decset(mouseButtonEvents))
 		_, _ = vx.tw.WriteString(decset(mouseAllEvents))
-		_, _ = vx.tw.WriteString(decset(mouseFocusEvents))
 		_, _ = vx.tw.WriteString(decset(mouseSGR))
+		if vx.caps.sgrPixels {
+			_, _ = vx.tw.WriteString(decset(mouseSGRPixels))
+		}
 	}
 	_, _ = vx.tw.Flush()
 }
@@ -1082,6 +1306,7 @@ func (vx *Vaxis) enableModes() {
 func (vx *Vaxis) disableModes() {
 	_, _ = vx.tw.WriteString(sgrReset)               // reset fg, bg, attrs
 	_, _ = vx.tw.WriteString(decrst(bracketedPaste)) // bracketed paste
+	_, _ = vx.tw.WriteString(decrst(mouseFocusEvents))
 	if vx.caps.kittyKeyboard {
 		_, _ = vx.tw.WriteString(kittyKBPop) // kitty keyboard
 	}
@@ -1090,13 +1315,16 @@ func (vx *Vaxis) disableModes() {
 	if !vx.disableMouse {
 		_, _ = vx.tw.WriteString(decrst(mouseButtonEvents))
 		_, _ = vx.tw.WriteString(decrst(mouseAllEvents))
-		_, _ = vx.tw.WriteString(decrst(mouseFocusEvents))
 		_, _ = vx.tw.WriteString(decrst(mouseSGR))
+
+		if vx.caps.sgrPixels {
+			_, _ = vx.tw.WriteString(decrst(mouseSGRPixels))
+		}
 	}
 	if vx.caps.sixels {
 		_, _ = vx.tw.WriteString(decrst(sixelScrolling))
 	}
-	if vx.caps.unicodeCore {
+	if vx.caps.unicodeCore && !vx.caps.explicitWidth {
 		_, _ = vx.tw.WriteString(decrst(unicodeCore))
 	}
 	if vx.caps.colorThemeUpdates {
@@ -1105,6 +1333,9 @@ func (vx *Vaxis) disableModes() {
 	if vx.caps.osc176 {
 		_, _ = vx.tw.WriteString(tparm(setAppID, vx.appIDLast))
 	}
+	if vx.caps.inBandResize {
+		_, _ = vx.tw.WriteString(decrst(inBandResize))
+	}
 	// Most terminals default to "text" mouse shape
 	_, _ = vx.tw.WriteString(tparm(mouseShape, MouseShapeTextInput))
 	_, _ = vx.tw.Flush()
@@ -1120,6 +1351,7 @@ func (vx *Vaxis) enterAltScreen() {
 func (vx *Vaxis) exitAltScreen() {
 	vx.HideCursor()
 	_, _ = vx.tw.WriteString(decset(cursorVisibility))
+	_, _ = vx.tw.WriteString(clear)
 	_, _ = vx.tw.WriteString(decrst(alternateScreen))
 	_, _ = vx.tw.Flush()
 }
@@ -1144,6 +1376,15 @@ func (vx *Vaxis) Suspend() error {
 
 	vx.disableModes()
 	vx.exitAltScreen()
+
+	// Reset to user value, or CursorDefault.
+	_, _ = vx.tw.WriteString(tparm(cursorStyleSet, int(vx.userCursorStyle)))
+	// Always show the cursor on exit
+	_, _ = vx.tw.WriteString(decset(cursorVisibility))
+	// Reset internal state to match reality
+	vx.cursorLast.style = vx.userCursorStyle
+
+	vx.tw.vx.tw.Flush()
 	signal.Stop(vx.chSigKill)
 	signal.Stop(vx.chSigWinSz)
 	vx.console.Reset()
@@ -1152,21 +1393,29 @@ func (vx *Vaxis) Suspend() error {
 
 // openTty opens the /dev/tty device, makes it raw, and starts an input parser
 func (vx *Vaxis) openTty(tgts []*os.File) error {
-	for _, s := range tgts {
-		if c, err := console.ConsoleFromFile(s); err == nil {
-			vx.console = c
-			break
+	if vx.withConsole != nil {
+		vx.console = vx.withConsole
+	} else {
+		for _, s := range tgts {
+			if c, err := console.ConsoleFromFile(s); err == nil {
+				vx.console = c
+				break
+			}
 		}
 	}
+
 	if vx.console == nil {
 		return console.ErrNotAConsole
 	}
+
 	err := vx.console.SetRaw()
 	if err != nil {
 		return err
 	}
+
 	vx.tw = newWriter(vx)
 	vx.parser = ansi.NewParser(vx.console)
+
 	go func() {
 		defer func() {
 			if err := recover(); err != nil {
@@ -1186,7 +1435,7 @@ func (vx *Vaxis) openTty(tgts []*os.File) error {
 				}
 			case <-vx.chSigWinSz:
 				atomicStore(&vx.resize, true)
-				vx.PostEvent(Redraw{})
+				vx.PostEventBlocking(Redraw{})
 			case <-vx.chSigKill:
 				vx.Close()
 				return
@@ -1200,21 +1449,30 @@ func (vx *Vaxis) openTty(tgts []*os.File) error {
 // and reenables input parsing. Upon resuming, a Resize event will be delivered.
 // It is entirely possible the terminal was resized while suspended.
 func (vx *Vaxis) Resume() error {
-	tgts := []*os.File{os.Stderr, os.Stdout, os.Stdin}
-	if vx.withTty != "" {
-		f, err := os.OpenFile(vx.withTty, os.O_RDWR, 0)
-		if err != nil {
-			return err
+	var tgts []*os.File
+	if vx.withConsole == nil {
+		tgts = []*os.File{os.Stderr, os.Stdout, os.Stdin}
+
+		if vx.withTty != "" {
+			f, err := os.OpenFile(vx.withTty, os.O_RDWR, 0)
+			if err != nil {
+				return err
+			}
+			tgts = []*os.File{f}
 		}
-		tgts = []*os.File{f}
 	}
+
 	err := vx.openTty(tgts)
 	if err != nil {
 		return err
 	}
+
 	vx.enterAltScreen()
 	vx.enableModes()
-	vx.setupSignals()
+
+	if !vx.noSignals {
+		vx.setupSignals()
+	}
 	atomicStore(&vx.resize, true)
 	return nil
 }
@@ -1273,10 +1531,6 @@ const (
 )
 
 func (vx *Vaxis) cursorStyle() string {
-	if vx.cursorNext.style == CursorDefault {
-		// Cursor block is the default
-		return tparm(cursorStyleSet, int(CursorBlock))
-	}
 	return tparm(cursorStyleSet, int(vx.cursorNext.style))
 }
 
@@ -1316,6 +1570,11 @@ func (vx *Vaxis) SetTitle(s string) {
 	_, _ = io.WriteString(vx.console, tparm(setTitle, s))
 }
 
+// SetPath sets the terminal's working directory via OSC 7
+func (vx *Vaxis) NotifyWorkingDirectory(s string) {
+	_, _ = io.WriteString(vx.console, tparm(setCWD, s))
+}
+
 // SetAppID sets the terminal's application ID via OSC 176
 func (vx *Vaxis) SetAppID(s string) {
 	_, _ = io.WriteString(vx.console, tparm(setAppID, s))
@@ -1349,10 +1608,11 @@ func (vx *Vaxis) advance(cell Cell) int {
 // This call can be expensive, callers should consider caching the result for
 // strings or characters which will need to be measured frequently
 func (vx *Vaxis) RenderedWidth(s string) int {
-	if vx.caps.unicodeCore {
+	if vx.caps.unicodeCore || vx.caps.explicitWidth {
 		return gwidth(s, unicodeStd)
 	}
 	if vx.caps.noZWJ {
+		log.Debug("nozwj")
 		return gwidth(s, noZWJ)
 	}
 	return gwidth(s, wcwidth)
@@ -1385,6 +1645,10 @@ func (vx *Vaxis) TerminalID() string {
 	return string(vx.termID)
 }
 
+func (vx *Vaxis) CanRGB() bool {
+	return vx.caps.rgb
+}
+
 func (vx *Vaxis) CanKittyGraphics() bool {
 	return vx.caps.kittyGraphics
 }
@@ -1393,6 +1657,14 @@ func (vx *Vaxis) CanSixel() bool {
 	return vx.caps.sixels
 }
 
+func (vx *Vaxis) CanReportColor() bool {
+	return vx.caps.osc4
+}
+
+func (vx *Vaxis) CanReportForegroundColor() bool {
+	return vx.caps.osc10
+}
+
 func (vx *Vaxis) CanReportBackgroundColor() bool {
 	return vx.caps.osc11
 }
@@ -1405,6 +1677,18 @@ func (vx *Vaxis) CanSetAppID() bool {
 	return vx.caps.osc176
 }
 
+func (vx *Vaxis) CanUnicodeCore() bool {
+	return vx.caps.unicodeCore
+}
+
+func (vx *Vaxis) CanExplicitWidth() bool {
+	return vx.caps.explicitWidth
+}
+
+func (vx *Vaxis) CanInBandResize() bool {
+	return vx.caps.inBandResize
+}
+
 func (vx *Vaxis) nextGraphicID() uint64 {
 	vx.graphicsIDNext += 1
 	return vx.graphicsIDNext
